//Dimension Engineering G meter project
//Written to be compiled on CCS PCWH 3.216
//Do not use for commercial purposes or we will send robots to destroy your homes

#include "Gmeter.h"

//since we don't change the input/output/highZ states
//of the pins at all, using fast IO will save some unnecessary instructions
#use fast_io(A)
#use fast_io(B)

#define CALIBRATED_0G 510 //this is the analog value for the zero G point
#define CALIBRATED_SENSITIVITY 19//this number will convert 10bit analog values
                                 //into Gs

//Character data for the 7 seg display. Not all characters get used in this project
#define ONE    0b11011101
#define TWO    0b10101000
#define THREE  0b10011000
#define FOUR   0b11010100
#define FIVE   0b10010010
#define SIX    0b10000010
#define SEVEN  0b11011001
#define EIGHT  0b10000000
#define NINE   0b11010000
#define ZERO   0b10000001
#define DP     0b01111111
#define A      0b11000000
#define B      0b10000110
#define C      0b10100011
#define D      0b10001100
#define E      0b10100010
#define F      0b11100010
#define G      0b10010000
#define BLANK  0b11111111
#define MINUS  0b11111110

//takes an array of three int8s
void update_display(int8* tripledigitpointer)
{
   int8 i;
   int8 digitarray[13]={ZERO,ONE,TWO,THREE,FOUR,FIVE,SIX,SEVEN,EIGHT,NINE,A,BLANK,C};
   int8 digitmask=0b01000000;  //A6 is digit1, A7 is 2, A0 is digit 3
   int8 output;
   for(i=0;i<3;i++)
   {
      output_a(digitmask);
      output=digitarray[tripledigitpointer[i]];
      if(i==0) output=output & DP; //put the decimal point on digit 1
      output_b(output);

      delay_ms(3);  //lights on most of the time
      output_b(0xff); //then turn them off to avoid blurring from last digit
      delay_us(10);
      digitmask=digitmask<<1; //shifts so next digit is lit
      if(digitmask==0) digitmask++; //for digit 3
   }
}

void main()
{

   int8 digits[3]={1,2,3};
   int16 analogvalue=0;
   int16 oldanalogvalue=0;
   signed int16 temp=0;
   signed int16 Gvalue=0;
   int8 i=0;

   setup_oscillator(OSC_4MHZ);
   setup_adc_ports(ALL_ANALOG);
   setup_adc(ADC_CLOCK_DIV_64);
   setup_spi(FALSE);
   setup_timer_0(RTCC_INTERNAL|RTCC_DIV_1);
   setup_timer_1(T1_DISABLED);
   setup_timer_2(T2_DISABLED,0,1);

   set_tris_a(0b00111110); //AN1-5 are inputs
   set_tris_b(0); // all outputs for 7 seg display

   while(1)
   {
      update_display(digits); //calling the display routine multiple times
      update_display(digits); //to slow down how often the data gets updated
      update_display(digits);
      update_display(digits);

      set_adc_channel(2); //(this is pin 1 on the PIC connected to DE-ACCM's Y output)
      delay_us(10);
      analogvalue=read_adc();
      delay_us(10);
      analogvalue+=read_adc();//take an average of 2 readings to reduce noise
      analogvalue = analogvalue / 2;

      temp = analogvalue-oldanalogvalue;

      if( temp>2 || temp<-2 ) //ignore small changes to make display more readable
      {
         oldanalogvalue=analogvalue;
         //convert analog value into G digits
         Gvalue=analogvalue-CALIBRATED_0G;
         Gvalue=Gvalue<<5;                     //left shifting then dividing allows you to perform
         Gvalue=Gvalue/CALIBRATED_SENSITIVITY; //more precise multiplication/division
         //now convert from the decimal place digits stored in Gvalue
         //to the format of an array of 3 integers
         i=2;
         while(i!=255)
         {
             digits[i]=(int8)(Gvalue%10);
             Gvalue=Gvalue/10;
             i--;
         }
      }
   }
}

